// File:  Complex.cpp  -  Illustrating Operator Overloading

#include <iostream.h>

/*==========================================================================*/
/*  Complex is a tiny portion of support for mathematical complex numbers   */
/*  Complex numbers have a real and an imaginary portion                    */
/*==========================================================================*/

class Complex
{
  public:
    Complex (void);
    Complex (const double dReal, const double dImag);
    Complex operator - (const Complex cRHS);
    double GetReal (void) const;
    double GetImag (void) const;

    friend Complex operator + (const Complex cLHS, const Complex cRHS);
    friend ostream& operator << (ostream& outs, const Complex cRHS);

  private:
    double dReal, dImag;
};


// prototype for the multiplication ordinary operator overload
Complex operator * (const Complex cLHS, const Complex cRHS);

/*==========================================================================*/
/*  the main routine -- which drives the world...                           */
/*==========================================================================*/

int main (void)
{
    cout << "Complex - Illustrating Operator Overloading" << endl << endl;

    Complex c1, c2 (3, 7), c3 (5, 4);

    cout << "c1 = " << c1 << endl;
    cout << "c2 = " << c2 << endl;
    cout << "c3 = " << c3 << endl;

    c1 = c2 + c3;
    cout << "c1 = c2 + c3 = " << c1 << endl;

    c1 = c2 - c3;
    cout << "c1 = c2 - c3 = " << c1 << endl;

    c1 = c2 * c3;
    cout << "c1 = c2 * c3 = " << c1 << endl;

    cout << endl << "*** Fin ***" << endl << endl;
    return 0;
}

/*==========================================================================*/
/*  Constructors for Complex, and two Accessors -- real and imaginary       */
/*==========================================================================*/

Complex::Complex (void)  {  dReal = dImag = 0;  }

Complex::Complex (const double dReal, const double dImag)
{
    this->dReal = dReal;  this->dImag = dImag;
}

double Complex::GetReal (void) const {  return dReal;  }

double Complex::GetImag (void) const {  return dImag;  }

/*==========================================================================*/
/*  Overload - as a member function                                         */
/*==========================================================================*/

Complex Complex::operator - (const Complex cRHS)
{
    return Complex (dReal - cRHS.dReal, dImag - cRHS.dImag);
}

/*==========================================================================*/
/*  Overload + as a friend function                                         */
/*==========================================================================*/

Complex operator + (const Complex cLHS, const Complex cRHS)
{
    return Complex (cLHS.dReal + cRHS.dReal, cLHS.dImag + cRHS.dImag);
}

/*==========================================================================*/
/*  Overload * as an ordinary function                                      */
/*==========================================================================*/

Complex operator * (const Complex cLHS, const Complex cRHS)
{
    double dReal = cLHS.GetReal() * cRHS.GetReal() - cLHS.GetImag() * cRHS.GetImag();
    double dImag = cLHS.GetReal() * cRHS.GetImag() + cLHS.GetImag() * cRHS.GetReal();

    return Complex (dReal, dImag);
}

/*==========================================================================*/
/*  Overload << as a friend function
/*==========================================================================*/

ostream& operator << (ostream& outs, const Complex cRHS)
{
    cout << "(" << cRHS.dReal << "," << cRHS.dImag << ")";
    return outs;
}

/*==========================================================================*/
/*==========================================================================*/


/* ..... Execution output .....

Complex - Illustrating Operator Overloading

c1 = (0,0)
c2 = (3,7)
c3 = (5,4)
c1 = c2 + c3 = (8,11)
c1 = c2 - c3 = (-2,3)
c1 = c2 * c3 = (-13,47)

*** Fin ***

..... End of output ..... */

